action = "synthesis"

syn_device = "xc7z020"
syn_grade = "-1"
syn_package = "clg484"
syn_top = "zedboard_top"
syn_project = "zedboard_top"
syn_tool = "vivado"

modules = {
  "local" : [ "../../../top/zedboard/verilog" ],
}


