action = "synthesis"

syn_device = "1k"
syn_grade = "dummy"
syn_package = "tq144"
syn_top = "icestick_top"
syn_project = "demo"
syn_tool = "icestorm"

modules = {
  "local" : [ "../../../top/icestick/verilog" ]
}

